@@ -4,31 +4,32 @@
 #       Copyright 1996-2000 by Sam Rushing
 #                                                All Rights Reserved.
 #
-
 RCS_ID =  '$Id: http_server.py,v 1.12 2004/04/21 15:11:44 akuchling Exp $'
 
 # python modules
-import os
 import re
 import socket
-import string
 import sys
 import time
 
+from supervisor.compat import as_bytes
+
 # async modules
-import asyncore_25 as asyncore
-import asynchat_25 as asynchat
+import supervisor.medusa.asyncore_25 as asyncore
+import supervisor.medusa.asynchat_25 as asynchat
 
 # medusa modules
-import http_date
-import producers
-import status_handler
-import logger
+import supervisor.medusa.http_date as http_date
+import supervisor.medusa.producers as producers
+import supervisor.medusa.logger as logger
 
 VERSION_STRING = RCS_ID.split()[2]
 
-from counter import counter
-from urllib import unquote, splitquery
+from supervisor.medusa.counter import counter
+try:
+    from urllib import unquote, splitquery
+except ImportError:
+    from urllib.parse import unquote, splitquery
 
 # ===========================================================================
 #                                                       Request Object
@@ -82,14 +83,17 @@ class http_request:
     def __getitem__ (self, key):
         return self.reply_headers[key]
 
+    def __contains__(self, key):
+        return key in self.reply_headers
+
     def has_key (self, key):
-        return self.reply_headers.has_key (key)
+        return key in self.reply_headers
 
     def build_reply_header (self):
-        return '\r\n'.join(
-                    [self.response(self.reply_code)] +
-                    map(lambda x: '%s: %s' % x, self.reply_headers.items())
-                ) + '\r\n\r\n'
+        header_items = ['%s: %s' % item for item in self.reply_headers.items()]
+        result = '\r\n'.join (
+            [self.response(self.reply_code)] + header_items) + '\r\n\r\n'
+        return as_bytes(result)
 
     ####################################################
     # multiple reply header management
@@ -124,19 +128,19 @@ class http_request:
         found_it = 0
 
         # Remove things from the old dict as well
-        if (self.reply_headers.has_key(name) and
+        if (name in self.reply_headers and
             (value is None or
              self.reply_headers[name] == value)):
             del self.reply_headers[name]
             found_it = 1
 
 
+        removed_headers = []
         if not value is None:
             if (name, value) in self.__reply_header_list:
                 removed_headers = [(name, value)]
                 found_it = 1
         else:
-            removed_headers = []
             for h in self.__reply_header_list:
                 if h[0] == name:
                     removed_headers.append(h)
@@ -186,7 +190,6 @@ class http_request:
 
         headers = [self.response(self.reply_code)]
         headers += ["%s: %s" % h for h in header_tuples]
-
         return '\r\n'.join(headers) + '\r\n\r\n'
 
     #---------------------------------------------------
@@ -224,7 +227,7 @@ class http_request:
     def get_header (self, header):
         header = header.lower()
         hc = self._header_cache
-        if not hc.has_key (header):
+        if header not in hc:
             h = header + ': '
             hl = len(h)
             for line in self.header:
@@ -260,11 +263,13 @@ class http_request:
                     )
 
     def push (self, thing):
-        if type(thing) == type(''):
-            self.outgoing.append(producers.simple_producer(thing,
-              buffer_size=len(thing)))
-        else:
-            self.outgoing.append(thing)
+        # Sometimes, text gets pushed by XMLRPC logic for later
+        # processing.
+        if isinstance(thing, str):
+            thing = as_bytes(thing)
+        if isinstance(thing, bytes):
+            thing = producers.simple_producer(thing, buffer_size=len(thing))
+        self.outgoing.append(thing)
 
     def response (self, code=200):
         message = self.responses[code]
@@ -278,17 +283,18 @@ class http_request:
                 'code': code,
                 'message': message,
                 }
+        s = as_bytes(s)
         self['Content-Length'] = len(s)
         self['Content-Type'] = 'text/html'
         # make an error reply
-        self.push (s)
+        self.push(s)
         self.done()
 
     # can also be used for empty replies
     reply_now = error
 
     def done (self):
-        "finalize this transaction - send output to the http channel"
+        """finalize this transaction - send output to the http channel"""
 
         # ----------------------------------------
         # persistent connection management
@@ -303,7 +309,7 @@ class http_request:
 
         if self.version == '1.0':
             if connection == 'keep-alive':
-                if not self.has_key ('Content-Length'):
+                if 'Content-Length' not in self:
                     close_it = 1
                 else:
                     self['Connection'] = 'Keep-Alive'
@@ -312,8 +318,8 @@ class http_request:
         elif self.version == '1.1':
             if connection == 'close':
                 close_it = 1
-            elif not self.has_key ('Content-Length'):
-                if self.has_key ('Transfer-Encoding'):
+            elif 'Content-Length' not in self:
+                if 'Transfer-Encoding' in self:
                     if not self['Transfer-Encoding'] == 'chunked':
                         close_it = 1
                 elif self.use_chunked:
@@ -449,6 +455,9 @@ class http_request:
              )
             )
 
+    def log_info(self, msg, level):
+        pass
+
 
 # ===========================================================================
 #                                                HTTP Channel Object
@@ -468,9 +477,10 @@ class http_channel (asynchat.async_chat):
         asynchat.async_chat.__init__ (self, conn)
         self.server = server
         self.addr = addr
-        self.set_terminator ('\r\n\r\n')
-        self.in_buffer = ''
+        self.set_terminator (b'\r\n\r\n')
+        self.in_buffer = b''
         self.creation_time = int (time.time())
+        self.last_used = self.creation_time
         self.check_maintenance()
 
     def __repr__ (self):
@@ -496,9 +506,9 @@ class http_channel (asynchat.async_chat):
 
     def kill_zombies (self):
         now = int (time.time())
-        for channel in asyncore.socket_map.values():
+        for channel in list(asyncore.socket_map.values()):
             if channel.__class__ == self.__class__:
-                if (now - channel.creation_time) > channel.zombie_timeout:
+                if (now - channel.last_used) > channel.zombie_timeout:
                     channel.close()
 
     # --------------------------------------------------
@@ -510,12 +520,14 @@ class http_channel (asynchat.async_chat):
     def send (self, data):
         result = asynchat.async_chat.send (self, data)
         self.server.bytes_out.increment (len(data))
+        self.last_used = int (time.time())
         return result
 
     def recv (self, buffer_size):
         try:
             result = asynchat.async_chat.recv (self, buffer_size)
             self.server.bytes_in.increment (len(result))
+            self.last_used = int (time.time())
             return result
         except MemoryError:
             # --- Save a Trip to Your Service Provider ---
@@ -553,8 +565,8 @@ class http_channel (asynchat.async_chat):
             self.current_request.found_terminator()
         else:
             header = self.in_buffer
-            self.in_buffer = ''
-            lines = header.split('\r\n')
+            self.in_buffer = b''
+            lines = header.split(b'\r\n')
 
             # --------------------------------------------------
             # crack the request header
@@ -742,10 +754,11 @@ class http_server (asyncore.dispatcher):
         self.handlers.remove (handler)
 
     def status (self):
+        from supervisor.medusa.util import english_bytes
         def nice_bytes (n):
-            return ''.join(status_handler.english_bytes(n))
+            return ''.join(english_bytes (n))
 
-        handler_stats = filter (None, map (maybe_status, self.handlers))
+        handler_stats = [_f for _f in map (maybe_status, self.handlers) if _f]
 
         if self.total_clients:
             ratio = self.total_requests.as_long() / float(self.total_clients.as_long())
@@ -760,7 +773,7 @@ class http_server (asyncore.dispatcher):
                          '<p><ul>'
                          '<li>Total <b>Clients:</b> %s'                 % self.total_clients,
                          '<b>Requests:</b> %s'                                  % self.total_requests,
-                         '<b>Requests/Client:</b> %.1f'                 % (ratio),
+                         '<b>Requests/Client:</b> %.1f'                 % ratio,
                          '<li>Total <b>Bytes In:</b> %s'        % (nice_bytes (self.bytes_in.as_long())),
                          '<b>Bytes Out:</b> %s'                         % (nice_bytes (self.bytes_out.as_long())),
                          '<li>Total <b>Exceptions:</b> %s'              % self.exceptions,
@@ -816,18 +829,15 @@ def crack_request (r):
         return None, None, None
 
 if __name__ == '__main__':
-    import sys
     if len(sys.argv) < 2:
-        print 'usage: %s <root> <port>' % (sys.argv[0])
+        print('usage: %s <root> <port>' % (sys.argv[0]))
     else:
-        import monitor
-        import filesys
-        import default_handler
-        import status_handler
-        import ftp_server
-        import chat_server
-        import resolver
-        import logger
+        import supervisor.medusa.monitor as monitor
+        import supervisor.medusa.filesys as filesys
+        import supervisor.medusa.default_handler as default_handler
+        import supervisor.medusa.ftp_server as ftp_server
+        import supervisor.medusa.chat_server as chat_server
+        import supervisor.medusa.resolver as resolver
         rs = resolver.caching_resolver ('127.0.0.1')
         lg = logger.file_logger (sys.stdout)
         ms = monitor.secure_monitor_server ('fnord', '127.0.0.1', 9999)
@@ -842,9 +852,7 @@ if __name__ == '__main__':
                 logger_object=lg
                 )
         cs = chat_server.chat_server ('', 7777)
-        sh = status_handler.status_extension([hs,ms,ftp,cs,rs])
-        hs.install_handler (sh)
-        if ('-p' in sys.argv):
+        if '-p' in sys.argv:
             def profile_loop ():
                 try:
                     asyncore.loop()
