@@ -4,7 +4,14 @@ import datetime
 import errno
 import types
 
-from supervisor.datatypes import signal_number
+from supervisor.compat import as_string
+from supervisor.compat import as_bytes
+from supervisor.compat import unicode
+
+from supervisor.datatypes import (
+    Automatic,
+    signal_number,
+    )
 
 from supervisor.options import readFile
 from supervisor.options import tailFile
@@ -19,8 +26,11 @@ from supervisor.events import notify
 from supervisor.events import RemoteCommunicationEvent
 
 from supervisor.http import NOT_DONE_YET
-from supervisor.xmlrpc import Faults
-from supervisor.xmlrpc import RPCError
+from supervisor.xmlrpc import (
+    capped_int,
+    Faults,
+    RPCError,
+    )
 
 from supervisor.states import SupervisorStates
 from supervisor.states import getSupervisorStateDescription
@@ -109,8 +119,8 @@ class SupervisorNamespaceRPCInterface:
             raise RPCError(Faults.NO_FILE, logfile)
 
         try:
-            return readFile(logfile, int(offset), int(length))
-        except ValueError, inst:
+            return as_string(readFile(logfile, int(offset), int(length)))
+        except ValueError as inst:
             why = inst.args[0]
             raise RPCError(getattr(Faults, why))
 
@@ -160,14 +170,24 @@ class SupervisorNamespaceRPCInterface:
 
     def reloadConfig(self):
         """
-        Reload configuration
+        Reload the configuration.
+
+        The result contains three arrays containing names of process
+        groups:
+
+        * `added` gives the process groups that have been added
+        * `changed` gives the process groups whose contents have
+          changed
+        * `removed` gives the process groups that are no longer
+          in the configuration
+
+        @return array result  [[added, changed, removed]]
 
-        @return boolean result  always return True unless error
         """
         self._update('reloadConfig')
         try:
             self.supervisord.options.process_config(do_usage=False)
-        except ValueError, msg:
+        except ValueError as msg:
             raise RPCError(Faults.CANT_REREAD, msg)
 
         added, changed, removed = self.supervisord.diff_to_active()
@@ -214,21 +234,21 @@ class SupervisorNamespaceRPCInterface:
         all_processes = []
 
         if lexical:
-            group_names = self.supervisord.process_groups.keys()
+            group_names = list(self.supervisord.process_groups.keys())
             group_names.sort()
             for group_name in group_names:
                 group = self.supervisord.process_groups[group_name]
-                process_names = group.processes.keys()
+                process_names = list(group.processes.keys())
                 process_names.sort()
                 for process_name in process_names:
                     process = group.processes[process_name]
                     all_processes.append((group, process))
         else:
-            groups = self.supervisord.process_groups.values()
+            groups = list(self.supervisord.process_groups.values())
             groups.sort() # asc by priority
 
             for group in groups:
-                processes = group.processes.values()
+                processes = list(group.processes.values())
                 processes.sort() # asc by priority
                 for process in processes:
                     all_processes.append((group, process))
@@ -270,9 +290,9 @@ class SupervisorNamespaceRPCInterface:
         # eventually fail
         try:
             filename, argv = process.get_execv_args()
-        except NotFound, why:
+        except NotFound as why:
             raise RPCError(Faults.NO_FILE, why.args[0])
-        except (NotExecutable, NoPermission), why:
+        except (NotExecutable, NoPermission) as why:
             raise RPCError(Faults.NOT_EXECUTABLE, why.args[0])
 
         if process.get_state() in RUNNING_STATES:
@@ -340,7 +360,7 @@ class SupervisorNamespaceRPCInterface:
         if group is None:
             raise RPCError(Faults.BAD_NAME, name)
 
-        processes = group.processes.values()
+        processes = list(group.processes.values())
         processes.sort()
         processes = [ (group, process) for process in processes ]
 
@@ -432,7 +452,7 @@ class SupervisorNamespaceRPCInterface:
         if group is None:
             raise RPCError(Faults.BAD_NAME, name)
 
-        processes = group.processes.values()
+        processes = list(group.processes.values())
         processes.sort()
         processes = [ (group, process) for process in processes ]
 
@@ -498,13 +518,14 @@ class SupervisorNamespaceRPCInterface:
         @param string signal  Signal to send, as name ('HUP') or number ('1')
         @return array
         """
-        self._update('signalProcessGroup')
 
         group = self.supervisord.process_groups.get(name)
+        self._update('signalProcessGroup')
+
         if group is None:
             raise RPCError(Faults.BAD_NAME, name)
 
-        processes = group.processes.values()
+        processes = list(group.processes.values())
         processes.sort()
         processes = [(group, process) for process in processes]
 
@@ -540,15 +561,39 @@ class SupervisorNamespaceRPCInterface:
         for gconfig in self.supervisord.options.process_group_configs:
             inuse = gconfig.name in self.supervisord.process_groups
             for pconfig in gconfig.process_configs:
-                configinfo.append(
-                    { 'name': pconfig.name,
-                      'group': gconfig.name,
-                      'inuse': inuse,
-                      'autostart': pconfig.autostart,
-                      'group_prio': gconfig.priority,
-                      'process_prio': pconfig.priority })
+                d = {'autostart': pconfig.autostart,
+                     'command': pconfig.command,
+                     'exitcodes': pconfig.exitcodes,
+                     'group': gconfig.name,
+                     'group_prio': gconfig.priority,
+                     'inuse': inuse,
+                     'killasgroup': pconfig.killasgroup,
+                     'name': pconfig.name,
+                     'process_prio': pconfig.priority,
+                     'redirect_stderr': pconfig.redirect_stderr,
+                     'startretries': pconfig.startretries,
+                     'startsecs': pconfig.startsecs,
+                     'stdout_capture_maxbytes': pconfig.stdout_capture_maxbytes,
+                     'stdout_events_enabled': pconfig.stdout_events_enabled,
+                     'stdout_logfile': pconfig.stdout_logfile,
+                     'stdout_logfile_backups': pconfig.stdout_logfile_backups,
+                     'stdout_logfile_maxbytes': pconfig.stdout_logfile_maxbytes,
+                     'stdout_syslog': pconfig.stdout_syslog,
+                     'stopsignal': int(pconfig.stopsignal), # enum on py3
+                     'stopwaitsecs': pconfig.stopwaitsecs,
+                     'stderr_capture_maxbytes': pconfig.stderr_capture_maxbytes,
+                     'stderr_events_enabled': pconfig.stderr_events_enabled,
+                     'stderr_logfile': pconfig.stderr_logfile,
+                     'stderr_logfile_backups': pconfig.stderr_logfile_backups,
+                     'stderr_logfile_maxbytes': pconfig.stderr_logfile_maxbytes,
+                     'stderr_syslog': pconfig.stderr_syslog,
+                    }
+                # no support for these types in xml-rpc
+                d.update((k, 'auto') for k, v in d.items() if v is Automatic)
+                d.update((k, 'none') for k, v in d.items() if v is None)
+                configinfo.append(d)
 
-        configinfo.sort()
+        configinfo.sort(key=lambda r: r['name'])
         return configinfo
 
     def _interpretProcessInfo(self, info):
@@ -595,9 +640,12 @@ class SupervisorNamespaceRPCInterface:
         if process is None:
             raise RPCError(Faults.BAD_NAME, name)
 
-        start = int(process.laststart)
-        stop = int(process.laststop)
-        now = int(time.time())
+        # TODO timestamps are returned as xml-rpc integers for b/c but will
+        # saturate the xml-rpc integer type in jan 2038 ("year 2038 problem").
+        # future api versions should return timestamps as a different type.
+        start = capped_int(process.laststart)
+        stop = capped_int(process.laststop)
+        now = capped_int(self._now())
 
         state = process.get_state()
         spawnerr = process.spawnerr or ''
@@ -625,6 +673,10 @@ class SupervisorNamespaceRPCInterface:
         info['description'] = description
         return info
 
+    def _now(self): # pragma: no cover
+        # this is here to service stubbing in unit tests
+        return time.time()
+
     def getAllProcessInfo(self):
         """ Get info about all processes
 
@@ -652,8 +704,8 @@ class SupervisorNamespaceRPCInterface:
             raise RPCError(Faults.NO_FILE, logfile)
 
         try:
-            return readFile(logfile, int(offset), int(length))
-        except ValueError, inst:
+            return as_string(readFile(logfile, int(offset), int(length)))
+        except ValueError as inst:
             why = inst.args[0]
             raise RPCError(getattr(Faults, why))
 
@@ -786,7 +838,7 @@ class SupervisorNamespaceRPCInterface:
             name = make_namespec(group.config.name, process.config.name)
             try:
                 callback(name)
-            except RPCError, e:
+            except RPCError as e:
                 results.append(
                     {'name':process.config.name,
                      'group':group.config.name,
@@ -824,12 +876,11 @@ class SupervisorNamespaceRPCInterface:
         """
         self._update('sendProcessStdin')
 
-        if isinstance(chars, unicode):
-            chars = chars.encode('utf-8')
-
-        if not isinstance(chars, basestring):
+        if not isinstance(chars, (str, bytes, unicode)):
             raise RPCError(Faults.INCORRECT_PARAMETERS, chars)
 
+        chars = as_bytes(chars)
+
         group, process = self._getGroupAndProcess(name)
 
         if process is None:
@@ -840,7 +891,7 @@ class SupervisorNamespaceRPCInterface:
 
         try:
             process.write(chars)
-        except OSError, why:
+        except OSError as why:
             if why.args[0] == errno.EPIPE:
                 raise RPCError(Faults.NO_FILE, name)
             else:
@@ -894,7 +945,7 @@ def make_allfunc(processes, predicate, func, **extra_kwargs):
                 if predicate(process):
                     try:
                         callback = func(name, **extra_kwargs)
-                    except RPCError, e:
+                    except RPCError as e:
                         results.append({'name':process.config.name,
                                         'group':group.config.name,
                                         'status':e.code,
@@ -919,7 +970,7 @@ def make_allfunc(processes, predicate, func, **extra_kwargs):
 
             try:
                 value = cb()
-            except RPCError, e:
+            except RPCError as e:
                 results.append(
                     {'name':process.config.name,
                      'group':group.config.name,
@@ -962,8 +1013,7 @@ def make_allfunc(processes, predicate, func, **extra_kwargs):
     return allfunc
 
 def isRunning(process):
-    if process.get_state() in RUNNING_STATES:
-        return True
+    return process.get_state() in RUNNING_STATES
 
 def isNotRunning(process):
     return not isRunning(process)
