@@ -1,15 +1,14 @@
 import base64
 import os
-import socket
 import stat
 import sys
+import socket
 import tempfile
 import unittest
 
-try:
-    from hashlib import sha1
-except ImportError:
-    from sha import new as sha1
+from supervisor.compat import as_bytes
+from supervisor.compat import as_string
+from supervisor.compat import sha1
 
 from supervisor.tests.base import DummySupervisor
 from supervisor.tests.base import PopulatedDummySupervisor
@@ -58,21 +57,22 @@ class LogtailHandlerTests(HandlerTests, unittest.TestCase):
         self.assertEqual(request._error, 410)
 
     def test_handle_request(self):
-        f = tempfile.NamedTemporaryFile()
-        t = f.name
-        options = DummyOptions()
-        pconfig = DummyPConfig(options, 'foo', 'foo', stdout_logfile=t)
-        supervisord = PopulatedDummySupervisor(options, 'foo', pconfig)
-        handler = self._makeOne(supervisord)
-        request = DummyRequest('/logtail/foo', None, None, None)
-        handler.handle_request(request)
-        self.assertEqual(request._error, None)
-        from supervisor.medusa import http_date
-        self.assertEqual(request.headers['Last-Modified'],
-                         http_date.build_http_date(os.stat(t)[stat.ST_MTIME]))
-        self.assertEqual(request.headers['Content-Type'], 'text/plain')
-        self.assertEqual(len(request.producers), 1)
-        self.assertEqual(request._done, True)
+        with tempfile.NamedTemporaryFile() as f:
+            t = f.name
+            options = DummyOptions()
+            pconfig = DummyPConfig(options, 'foo', 'foo', stdout_logfile=t)
+            supervisord = PopulatedDummySupervisor(options, 'foo', pconfig)
+            handler = self._makeOne(supervisord)
+            request = DummyRequest('/logtail/foo', None, None, None)
+            handler.handle_request(request)
+            self.assertEqual(request._error, None)
+            from supervisor.medusa import http_date
+            self.assertEqual(request.headers['Last-Modified'],
+                http_date.build_http_date(os.stat(t)[stat.ST_MTIME]))
+            self.assertEqual(request.headers['Content-Type'], 'text/plain;charset=utf-8')
+            self.assertEqual(request.headers['X-Accel-Buffering'], 'no')
+            self.assertEqual(len(request.producers), 1)
+            self.assertEqual(request._done, True)
 
 class MainLogTailHandlerTests(HandlerTests, unittest.TestCase):
     def _getTargetClass(self):
@@ -96,19 +96,19 @@ class MainLogTailHandlerTests(HandlerTests, unittest.TestCase):
 
     def test_handle_request(self):
         supervisor = DummySupervisor()
-        f = tempfile.NamedTemporaryFile()
-        t = f.name
-        supervisor.options.logfile = t
-        handler = self._makeOne(supervisor)
-        request = DummyRequest('/mainlogtail', None, None, None)
-        handler.handle_request(request)
-        self.assertEqual(request._error, None)
-        from supervisor.medusa import http_date
-        self.assertEqual(request.headers['Last-Modified'],
-                         http_date.build_http_date(os.stat(t)[stat.ST_MTIME]))
-        self.assertEqual(request.headers['Content-Type'], 'text/plain')
-        self.assertEqual(len(request.producers), 1)
-        self.assertEqual(request._done, True)
+        with tempfile.NamedTemporaryFile() as f:
+            t = f.name
+            supervisor.options.logfile = t
+            handler = self._makeOne(supervisor)
+            request = DummyRequest('/mainlogtail', None, None, None)
+            handler.handle_request(request)
+            self.assertEqual(request._error, None)
+            from supervisor.medusa import http_date
+            self.assertEqual(request.headers['Last-Modified'],
+                http_date.build_http_date(os.stat(t)[stat.ST_MTIME]))
+            self.assertEqual(request.headers['Content-Type'], 'text/plain;charset=utf-8')
+            self.assertEqual(len(request.producers), 1)
+            self.assertEqual(request._done, True)
 
 
 class TailFProducerTests(unittest.TestCase):
@@ -123,15 +123,15 @@ class TailFProducerTests(unittest.TestCase):
         request = DummyRequest('/logtail/foo', None, None, None)
         from supervisor import http
         f = tempfile.NamedTemporaryFile()
-        f.write('a' * 80)
+        f.write(b'a' * 80)
         f.flush()
         producer = self._makeOne(request, f.name, 80)
         result = producer.more()
-        self.assertEqual(result, 'a' * 80)
-        f.write('w' * 100)
+        self.assertEqual(result, b'a' * 80)
+        f.write(as_bytes(b'w' * 100))
         f.flush()
         result = producer.more()
-        self.assertEqual(result, 'w' * 100)
+        self.assertEqual(result, b'w' * 100)
         result = producer.more()
         self.assertEqual(result, http.NOT_DONE_YET)
         f.truncate(0)
@@ -141,50 +141,48 @@ class TailFProducerTests(unittest.TestCase):
 
     def test_handle_more_fd_closed(self):
         request = DummyRequest('/logtail/foo', None, None, None)
-        f = tempfile.NamedTemporaryFile()
-        f.write('a' * 80)
-        f.flush()
-        producer = self._makeOne(request, f.name, 80)
-        producer.file.close()
-        result = producer.more()
+        with tempfile.NamedTemporaryFile() as f:
+            f.write(as_bytes('a' * 80))
+            f.flush()
+            producer = self._makeOne(request, f.name, 80)
+            producer.file.close()
+            result = producer.more()
         self.assertEqual(result, producer.more())
 
     def test_handle_more_follow_file_recreated(self):
         request = DummyRequest('/logtail/foo', None, None, None)
         f = tempfile.NamedTemporaryFile()
-        f.write('a' * 80)
+        f.write(as_bytes('a' * 80))
         f.flush()
         producer = self._makeOne(request, f.name, 80)
         result = producer.more()
-        self.assertEqual(result, 'a' * 80)
+        self.assertEqual(result, b'a' * 80)
         f.close()
-        f2 = open(f.name, 'w')
+        f2 = open(f.name, 'wb')
         try:
-            f2.write('b' * 80)
+            f2.write(as_bytes(b'b' * 80))
             f2.close()
             result = producer.more()
         finally:
             os.unlink(f2.name)
-        self.assertEqual(result, 'b' * 80)
+        self.assertEqual(result, b'b' * 80)
 
     def test_handle_more_follow_file_gone(self):
         request = DummyRequest('/logtail/foo', None, None, None)
         filename = tempfile.mktemp()
-        f = open(filename, 'wb')
-        f.write('a' * 80)
-        f.close()
+        with open(filename, 'wb') as f:
+            f.write(b'a' * 80)
         try:
             producer = self._makeOne(request, f.name, 80)
         finally:
             os.unlink(f.name)
         result = producer.more()
-        self.assertEqual(result, 'a' * 80)
-        f = open(filename, 'wb')
-        f.write('b' * 80)
-        f.close()
+        self.assertEqual(result, b'a' * 80)
+        with open(filename, 'wb') as f:
+            f.write(as_bytes(b'b' * 80))
         try:
             result = producer.more() # should open in new file
-            self.assertEqual(result, 'b' * 80)
+            self.assertEqual(result, b'b' * 80)
         finally:
              os.unlink(f.name)
 
@@ -202,28 +200,28 @@ class DeferringChunkedProducerTests(unittest.TestCase):
         self.assertEqual(producer.more(), NOT_DONE_YET)
 
     def test_more_string(self):
-        wrapped = DummyProducer('hello')
+        wrapped = DummyProducer(b'hello')
         producer = self._makeOne(wrapped)
-        self.assertEqual(producer.more(), '5\r\nhello\r\n')
+        self.assertEqual(producer.more(), b'5\r\nhello\r\n')
 
     def test_more_nodata(self):
         wrapped = DummyProducer()
-        producer = self._makeOne(wrapped, footers=['a', 'b'])
-        self.assertEqual(producer.more(), '0\r\na\r\nb\r\n\r\n')
+        producer = self._makeOne(wrapped, footers=[b'a', b'b'])
+        self.assertEqual(producer.more(), b'0\r\na\r\nb\r\n\r\n')
 
     def test_more_nodata_footers(self):
-        wrapped = DummyProducer('')
-        producer = self._makeOne(wrapped, footers=['a', 'b'])
-        self.assertEqual(producer.more(), '0\r\na\r\nb\r\n\r\n')
+        wrapped = DummyProducer(b'')
+        producer = self._makeOne(wrapped, footers=[b'a', b'b'])
+        self.assertEqual(producer.more(), b'0\r\na\r\nb\r\n\r\n')
 
     def test_more_nodata_nofooters(self):
-        wrapped = DummyProducer('')
+        wrapped = DummyProducer(b'')
         producer = self._makeOne(wrapped)
-        self.assertEqual(producer.more(), '0\r\n\r\n')
+        self.assertEqual(producer.more(), b'0\r\n\r\n')
 
     def test_more_noproducer(self):
         producer = self._makeOne(None)
-        self.assertEqual(producer.more(), '')
+        self.assertEqual(producer.more(), b'')
 
 class DeferringCompositeProducerTests(unittest.TestCase):
     def _getTargetClass(self):
@@ -244,12 +242,12 @@ class DeferringCompositeProducerTests(unittest.TestCase):
         producer = self._makeOne([wrapped1, wrapped2])
         self.assertEqual(producer.more(), 'hello')
         self.assertEqual(producer.more(), 'goodbye')
-        self.assertEqual(producer.more(), '')
+        self.assertEqual(producer.more(), b'')
 
     def test_more_nodata(self):
         wrapped = DummyProducer()
         producer = self._makeOne([wrapped])
-        self.assertEqual(producer.more(), '')
+        self.assertEqual(producer.more(), b'')
 
 class DeferringGlobbingProducerTests(unittest.TestCase):
     def _getTargetClass(self):
@@ -267,16 +265,16 @@ class DeferringGlobbingProducerTests(unittest.TestCase):
     def test_more_string(self):
         wrapped = DummyProducer('hello', 'there', 'guy')
         producer = self._makeOne(wrapped, buffer_size=1)
-        self.assertEqual(producer.more(), 'hello')
+        self.assertEqual(producer.more(), b'hello')
 
         wrapped = DummyProducer('hello', 'there', 'guy')
         producer = self._makeOne(wrapped, buffer_size=50)
-        self.assertEqual(producer.more(), 'hellothereguy')
+        self.assertEqual(producer.more(), b'hellothereguy')
 
     def test_more_nodata(self):
         wrapped = DummyProducer()
         producer = self._makeOne(wrapped)
-        self.assertEqual(producer.more(), '')
+        self.assertEqual(producer.more(), b'')
 
 class DeferringHookedProducerTests(unittest.TestCase):
     def _getTargetClass(self):
@@ -308,12 +306,12 @@ class DeferringHookedProducerTests(unittest.TestCase):
         def callback(bytes):
             L.append(bytes)
         producer = self._makeOne(wrapped, callback)
-        self.assertEqual(producer.more(), '')
+        self.assertEqual(producer.more(), b'')
         self.assertEqual(L, [0])
 
     def test_more_noproducer(self):
         producer = self._makeOne(None, None)
-        self.assertEqual(producer.more(), '')
+        self.assertEqual(producer.more(), b'')
 
 class DeferringHttpRequestTests(unittest.TestCase):
     def _getTargetClass(self):
@@ -507,12 +505,12 @@ class EncryptedDictionaryAuthorizedTests(unittest.TestCase):
         self.assertTrue(authorizer.authorize(('foo', 'pass:word')))
 
     def test_authorize_gooduser_badpassword_sha(self):
-        password = '{SHA}' + sha1('password').hexdigest()
+        password = '{SHA}' + sha1(as_bytes('password')).hexdigest()
         authorizer = self._makeOne({'foo':password})
         self.assertFalse(authorizer.authorize(('foo', 'bar')))
 
     def test_authorize_gooduser_goodpassword_sha(self):
-        password = '{SHA}' + sha1('password').hexdigest()
+        password = '{SHA}' + sha1(as_bytes('password')).hexdigest()
         authorizer = self._makeOne({'foo':password})
         self.assertTrue(authorizer.authorize(('foo', 'password')))
 
@@ -532,8 +530,8 @@ class SupervisorAuthHandlerTests(unittest.TestCase):
 
     def test_handle_request_authorizes_good_credentials(self):
         request = DummyRequest('/logtail/process1', None, None, None)
-        encoded = base64.b64encode("user:password")
-        request.header = ["Authorization: Basic %s" % encoded]
+        encoded = base64.b64encode(as_bytes("user:password"))
+        request.header = ["Authorization: Basic %s" % as_string(encoded)]
         handler = DummyHandler()
         auth_handler = self._makeOne({'user':'password'}, handler)
         auth_handler.handle_request(request)
@@ -541,8 +539,9 @@ class SupervisorAuthHandlerTests(unittest.TestCase):
 
     def test_handle_request_authorizes_good_password_with_colon(self):
         request = DummyRequest('/logtail/process1', None, None, None)
-        encoded = base64.b64encode("user:pass:word") # password contains colon
-        request.header = ["Authorization: Basic %s" % encoded]
+        # password contains colon
+        encoded = base64.b64encode(as_bytes("user:pass:word"))
+        request.header = ["Authorization: Basic %s" % as_string(encoded)]
         handler = DummyHandler()
         auth_handler = self._makeOne({'user':'pass:word'}, handler)
         auth_handler.handle_request(request)
@@ -550,8 +549,8 @@ class SupervisorAuthHandlerTests(unittest.TestCase):
 
     def test_handle_request_does_not_authorize_bad_credentials(self):
         request = DummyRequest('/logtail/process1', None, None, None)
-        encoded = base64.b64encode("wrong:wrong")
-        request.header = ["Authorization: Basic %s" % encoded]
+        encoded = base64.b64encode(as_bytes("wrong:wrong"))
+        request.header = ["Authorization: Basic %s" % as_string(encoded)]
         handler = DummyHandler()
         auth_handler = self._makeOne({'user':'password'}, handler)
         auth_handler.handle_request(request)
@@ -615,16 +614,16 @@ class TopLevelFunctionTests(unittest.TestCase):
                   'username':None, 'password':None,
                   'section':'inet_http_server'}
         try:
-            servers = self._make_http_servers([config])
+            self._make_http_servers([config])
             self.fail('nothing raised')
-        except ValueError, exc:
+        except ValueError as exc:
             self.assertEqual(exc.args[0], 'Cannot determine socket type 999')
 
     def test_make_http_servers_noauth(self):
         socketfile = tempfile.mktemp()
         inet = {'family':socket.AF_INET, 'host':'localhost', 'port':17735,
                 'username':None, 'password':None, 'section':'inet_http_server'}
-        unix = {'family':socket.AF_UNIX, 'file':socketfile, 'chmod':int('700', 8),
+        unix = {'family':socket.AF_UNIX, 'file':socketfile, 'chmod':0o700,
                 'chown':(-1, -1), 'username':None, 'password':None,
                 'section':'unix_http_server'}
         servers = self._make_http_servers([inet, unix])
@@ -652,7 +651,7 @@ class TopLevelFunctionTests(unittest.TestCase):
         inet = {'family':socket.AF_INET, 'host':'localhost', 'port':17736,
                 'username':'username', 'password':'password',
                 'section':'inet_http_server'}
-        unix = {'family':socket.AF_UNIX, 'file':socketfile, 'chmod':int('700', 8),
+        unix = {'family':socket.AF_UNIX, 'file':socketfile, 'chmod':0o700,
                 'chown':(-1, -1), 'username':'username', 'password':'password',
                 'section':'unix_http_server'}
         servers = self._make_http_servers([inet, unix])
@@ -678,7 +677,7 @@ class DummyProducer:
         if self.data:
             return self.data.pop(0)
         else:
-            return ''
+            return b''
 
 def test_suite():
     return unittest.findTestCases(sys.modules[__name__])
